@@ -1,6 +1,5 @@
 (require 'magit)
 
-(require 'assoc)
 (eval-when-compile (require 'cl))
 
 (defvar magit-key-mode-key-maps '()
@@ -82,13 +81,20 @@
     (branching
      (man-page "git-branch")
      (actions
-      ("v" "Branch manager" magit-show-branches)
-      ("n" "New" magit-create-branch)
-      ("m" "Move" magit-move-branch)
-      ("d" "Delete" magit-delete-branch)
-      ("D" "Force Delete" magit-delete-branch-forced)
+      ("v" "Branch manager" magit-branch-manager)
+      ("c" "Create" magit-create-branch)
+      ("r" "Rename" magit-move-branch)
+      ("k" "Delete" magit-delete-branch)
       ("b" "Checkout" magit-checkout)))
 
+    (remoting
+     (man-page "git-remote")
+     (actions
+      ("v" "Branch manager" magit-branch-manager)
+      ("a" "Add" magit-add-remote)
+      ("r" "Rename" magit-rename-remote)
+      ("k" "Remove" magit-remove-remote)))
+
     (tagging
      (man-page "git-tag")
      (actions
@@ -103,16 +109,17 @@
       ("z" "Save" magit-stash)
       ("s" "Snapshot" magit-stash-snapshot))
      (switches
-      ("-k" "Keep index" "--keep-index")))
+      ("-k" "Keep index" "--keep-index")
+      ("-u" "Include untracked files" "--include-untracked")
+      ("-a" "Include all files" "--all")))
 
     (merging
      (man-page "git-merge")
      (actions
-      ("m" "Merge" magit-merge))
+      ("m" "Merge" magit-manual-merge))
      (switches
       ("-ff" "Fast-forward only" "--ff-only")
       ("-nf" "No fast-forward" "--no-ff")
-      ("-nc" "No commit" "--no-commit")
       ("-sq" "Squash" "--squash"))
      (arguments
       ("-st" "Strategy" "--strategy=" read-from-minibuffer)))
@@ -169,7 +176,7 @@ and put in its place an empty one of the same name."
   (when (assoc group magit-key-mode-groups)
     (magit-key-mode-delete-group group))
   (setq magit-key-mode-groups
-        (cons (list group '(actions)) magit-key-mode-groups)))
+        (cons (list group (list 'actions)) magit-key-mode-groups)))
 
 (defun magit-key-mode-key-defined-p (for-group key)
   "If KEY is defined as any of switch, argument or action within
@@ -244,6 +251,17 @@ FOR-GROUP."
                   (error "Nothing at point to do.")))
          (def (lookup-key (current-local-map) key)))
     (call-interactively def)))
+(defun magit-key-mode-jump-to-next-exec ()
+  "Jump to the next action/args/option point."
+  (interactive)
+  (let* ((oldp (point))
+         (old  (get-text-property oldp 'key-group-executor))
+         (p    (if (= oldp (point-max)) (point-min) (1+ oldp))))
+    (while (let ((new (get-text-property p 'key-group-executor)))
+             (and (not (= p oldp)) (or (not new) (eq new old))))
+      (setq p (if (= p (point-max)) (point-min) (1+ p))))
+    (goto-char p)
+    (skip-chars-forward " ")))
 
 (defun magit-key-mode-build-keymap (for-group)
   "Construct a normal looking keymap for the key mode to use and
@@ -256,6 +274,8 @@ put it in magit-key-mode-key-maps for fast lookup."
     (suppress-keymap map 'nodigits)
     ;; ret dwim
     (define-key map (kbd "RET") 'magit-key-mode-exec-at-point)
+    ;; tab jumps to the next "button"
+    (define-key map (kbd "TAB") 'magit-key-mode-jump-to-next-exec)
 
     ;; all maps should `quit' with `C-g' or `q'
     (define-key map (kbd "C-g") `(lambda ()
@@ -289,7 +309,7 @@ put it in magit-key-mode-key-maps for fast lookup."
           (defkey k `(magit-key-mode-add-argument
                       ',for-group ,(nth 2 k) ',(nth 3 k))))))
 
-    (aput 'magit-key-mode-key-maps for-group map)
+    (push (cons for-group map) magit-key-mode-key-maps)
     map))
 
 (defvar magit-key-mode-prefix nil
@@ -376,20 +396,25 @@ highlighted before the description."
 (defun magit-key-mode-redraw (for-group)
   "(re)draw the magit key buffer."
   (let ((buffer-read-only nil)
-        (old-point (point)))
+        (old-point (point))
+        (is-first (zerop (buffer-size)))
+        (actions-p nil))
     (erase-buffer)
     (make-local-variable 'font-lock-defaults)
     (use-local-map (magit-key-mode-get-key-map for-group))
-    (magit-key-mode-draw for-group)
+    (setq actions-p (magit-key-mode-draw for-group))
     (delete-trailing-whitespace)
     (setq mode-name "magit-key-mode" major-mode 'magit-key-mode)
-    (goto-char old-point))
+    (if (and is-first actions-p)
+      (progn (goto-char actions-p)
+             (magit-key-mode-jump-to-next-exec))
+      (goto-char old-point)))
   (setq buffer-read-only t)
   (fit-window-to-buffer))
 
 (defun magit-key-mode-draw-header (header)
   "Draw a header with the correct face."
-  (insert (propertize header 'face 'font-lock-keyword-face)))
+  (insert (propertize header 'face 'font-lock-keyword-face) "\n"))
 
 (defvar magit-key-mode-args-in-cols nil
   "When true, draw arguments in columns as with switches and
@@ -397,64 +422,47 @@ highlighted before the description."
 
 (defun magit-key-mode-draw-args (args)
   "Draw the args part of the menu."
-  (when args
-    (let ((strs (mapcar
-                 (lambda (argument)
-                   (propertize
-                    (format " %s: %s (%s) %s"
-                            (propertize
-                             (car argument)
-                             'face 'font-lock-builtin-face)
-                            (nth 1 argument)
-                            (nth 2 argument)
-                            (propertize
-                             (gethash (nth 2 argument)
-                                      magit-key-mode-current-args
-                                      "")
-                             'face 'widget-field))
-                    'key-group-executor (car argument)))
-                 args)))
-      (magit-key-mode-draw-header "Args\n")
-      (magit-key-mode-draw-in-cols strs (not magit-key-mode-args-in-cols)))))
+  (magit-key-mode-draw-buttons
+   "Args"
+   args
+   (lambda (x)
+     (format "(%s) %s"
+             (nth 2 x)
+             (propertize (gethash (nth 2 x) magit-key-mode-current-args "")
+                         'face 'widget-field)))
+   (not magit-key-mode-args-in-cols)))
 
 (defun magit-key-mode-draw-switches (switches)
   "Draw the switches part of the menu."
-  (when switches
-    (let ((switch-strs (mapcar
-                        (lambda (s)
-                          (let ((option (nth 2 s)))
-                            (propertize
-                             (format " %s: %s (%s)"
-                                     (propertize (car s)
-                                                 'face 'font-lock-builtin-face)
-                                     (nth 1 s)
-                                     (if (member option magit-key-mode-current-options)
-                                         (propertize
-                                          option
-                                          'face 'font-lock-warning-face)
-                                       option))
-                             'key-group-executor (car s))))
-                        switches)))
-      (magit-key-mode-draw-header "Switches\n")
-      (magit-key-mode-draw-in-cols switch-strs))))
+  (magit-key-mode-draw-buttons
+   "Switches"
+   switches
+   (lambda (x)
+     (format "(%s)" (let ((s (nth 2 x)))
+                      (if (member s magit-key-mode-current-options)
+                        (propertize s 'face 'font-lock-warning-face)
+                        s))))))
 
 (defun magit-key-mode-draw-actions (actions)
   "Draw the actions part of the menu."
-  (when actions
-    (let ((action-strs (mapcar
-                        (lambda (a)
-                          (propertize
-                           (format
-                            " %s: %s"
-                            (propertize (car a)
-                                        'face 'font-lock-builtin-face)
-                            (nth 1 a))
-                           'key-group-executor (car a)))
-                       actions)))
-    (magit-key-mode-draw-header "Actions\n")
-    (magit-key-mode-draw-in-cols action-strs))))
+  (magit-key-mode-draw-buttons "Actions" actions nil))
 
-(defun magit-key-mode-draw-in-cols (strings &optional one-col-each)
+(defun magit-key-mode-draw-buttons (section xs maker
+                                    &optional one-col-each)
+  (when xs
+    (magit-key-mode-draw-header section)
+    (magit-key-mode-draw-in-cols
+     (mapcar (lambda (x)
+               (let* ((head (propertize (car x) 'face 'font-lock-builtin-face))
+                      (desc (nth 1 x))
+                      (more (and maker (funcall maker x)))
+                      (text (format " %s: %s%s%s"
+                                    head desc (if more " " "") (or more ""))))
+                 (propertize text 'key-group-executor (car x))))
+             xs)
+     one-col-each)))
+
+(defun magit-key-mode-draw-in-cols (strings one-col-each)
   "Given a list of strings, print in columns (using `insert'). If
 ONE-COL-EACH is true then don't columify, but rather, draw each
 item on one line."
@@ -475,15 +483,20 @@ item on one line."
   (insert "\n"))
 
 (defun magit-key-mode-draw (for-group)
-  "Function used to draw actions, switches and parameters."
+  "Function used to draw actions, switches and parameters.
+
+Returns the point before the actions part, if any."
   (let* ((options (magit-key-mode-options-for-group for-group))
          (switches (cdr (assoc 'switches options)))
          (arguments (cdr (assoc 'arguments options)))
-         (actions (cdr (assoc 'actions options))))
+         (actions (cdr (assoc 'actions options)))
+         (p nil))
     (magit-key-mode-draw-switches switches)
     (magit-key-mode-draw-args arguments)
+    (when actions (setq p (point-marker)))
     (magit-key-mode-draw-actions actions)
-    (insert "\n")))
+    (insert "\n")
+    p))
 
 (defun magit-key-mode-de-generate (group)
   "Unbind the function for GROUP."
